package ai.zowie_flutter_sdk

import ai.zowie.Zowie
import ai.zowie.ZowieAuthenticationType
import ai.zowie.ZowieColor
import ai.zowie.ZowieColors
import ai.zowie.ZowieConfiguration
import ai.zowie.ZowieMetadata
import ai.zowie.ZowieOnErrorListener
import ai.zowie.ZowieOnSuccessListener
import ai.zowie.ZowieStrings
import android.app.Activity
import android.app.Application
import android.content.Intent
import android.graphics.Color
import androidx.annotation.NonNull
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import org.json.JSONObject
import java.lang.ref.WeakReference

class ZowieFlutterSdkPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {

    private lateinit var channel: MethodChannel
    private val gson = Gson()
    private var activity: WeakReference<Activity>? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, CHANNEL)
        channel.setMethodCallHandler(this)

        Zowie.initialize(flutterPluginBinding.applicationContext as Application)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            METHOD_CONFIGURATION -> onConfiguration(call, result)
            METHOD_SHOW_CHAT -> onShowChat(result)
            METHOD_METADATA -> onMetadata(call, result)
            METHOD_ENABLE_NOTIFICATIONS -> onEnableNotifications(call, result)
            METHOD_DISABLE_NOTIFICATIONS -> onDisableNotifications(result)
            METHOD_SET_USER_STATUS -> onSetUserStatus(call, result)
            METHOD_SET_CONTEXT -> onSetContext(call, result)
            METHOD_LOCALIZATION -> onLocalization(call, result)
            METHOD_COLORS -> onColors(call, result)
            else -> result.notImplemented()
        }
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = WeakReference(binding.activity)
    }

    override fun onDetachedFromActivityForConfigChanges() = Unit

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) = Unit

    override fun onDetachedFromActivity() {
        activity?.clear()
        activity = null
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    private fun onConfiguration(call: MethodCall, result: Result) {
        val jwt = call.argument<String>("jwt")
        val configuration = ZowieConfiguration {
            userId = call.argument("userId")
            conversationId = call.argument("conversationId")
            instanceId = call.argument("instanceId")
            conversationInitReferral = call.argument("conversationInitReferral")
            authenticationType = if (jwt != null && jwt.isNotEmpty()) {
                ZowieAuthenticationType.JwtToken(jwt)
            } else {
                ZowieAuthenticationType.Raw
            }
        }

        Zowie.setConfiguration(configuration)

        result.success()
    }

    private fun onShowChat(result: Result) {
        activity?.get()?.let {
            it.startActivity(Intent(it, ChatActivity::class.java))
            result.success()
        } ?: run {
            result.error("onShowChat", "activity is null")
        }
    }

    private fun onMetadata(call: MethodCall, result: Result) {
        val extraParamsResult = if (call.hasArgument("extraParams")) {
            kotlin.runCatching {
                val type = object : TypeToken<Map<String, String>>() {}.type
                val json = call.argument<String>("extraParams")

                return@runCatching gson.fromJson<Map<String, String>>(json, type)
            }.getOrNull()
        } else {
            null
        }
        val metadata = ZowieMetadata {
            firstName = call.argument("firstName")
            lastName = call.argument("lastName")
            name = call.argument("name")
            locale = call.argument("locale")
            timeZone = call.argument("timeZone")
            phoneNumber = call.argument("phoneNumber")
            email = call.argument("email")
            extraParams = extraParamsResult
        }

        Zowie.setMetadata(
                metadata = metadata,
                onErrorListener = getErrorListener(result, "onMetadata"),
                onSuccessListener = getSuccessListener(result)
        )
    }

    private fun onEnableNotifications(call: MethodCall, result: Result) {
        Zowie.enableNotifications(
                fcmToken = call.argument("fcmToken")!!,
                onErrorListener = getErrorListener(result, "onEnableNotifications"),
                onSuccessListener = getSuccessListener(result)
        )
    }

    private fun onDisableNotifications(result: Result) {
        Zowie.disableNotifications(
                onErrorListener = getErrorListener(result, "onDisableNotifications"),
                onSuccessListener = getSuccessListener(result)
        )
    }

    private fun onSetUserStatus(call: MethodCall, result: Result) {
        Zowie.setUserStatus(
                isActive = call.argument("isActive")!!,
                onErrorListener = getErrorListener(result, "onSetUserStatus"),
                onSuccessListener = getSuccessListener(result)
        )
    }

    private fun onSetContext(call: MethodCall, result: Result) {
        Zowie.setContext(
                contextId = call.argument("contextId")!!,
                onErrorListener = getErrorListener(result, "onSetContext"),
                onSuccessListener = getSuccessListener(result)
        )
    }

    private fun onLocalization(call: MethodCall, result: Result) {
        val zowieStrings = ZowieStrings {
            newMessageHint = call.argument("newMessageHint")
            messageStatusDelivered = call.argument("messageStatusDelivered")
            messageStatusSendingErrorMessage = call.argument("messageStatusSendingErrorMessage")
            messageStatusSendingErrorTryAgain = call.argument("messageStatusSendingErrorTryAgain")
            readAndWriteStoragePermissionAlertTitle = call.argument("readAndWriteStoragePermissionAlertTitle")
            readAndWriteStoragePermissionAlertMessage = call.argument("readAndWriteStoragePermissionAlertMessage")
            readAndWriteStoragePermissionAlertPositiveButton = call.argument("readAndWriteStoragePermissionAlertPositiveButton")
            readAndWriteStoragePermissionAlertNegativeButton = call.argument("readAndWriteStoragePermissionAlertNegativeButton")
            attachmentPlaceholderName = call.argument("attachmentPlaceholderName")
            attachmentFileMaxSizeExceededErrorMessage = call.argument("attachmentFileMaxSizeExceededErrorMessage")
            couldNotOpenFileErrorMessage = call.argument("couldNotOpenFileErrorMessage")
            chatConnectionErrorMessage = call.argument("chatConnectionErrorMessage")
            chatConnectionRestoredMessage = call.argument("chatConnectionRestoredMessage")
            chatHistoryDownloadErrorMessage = call.argument("chatHistoryDownloadErrorMessage")
            couldNotOpenWebBrowserErrorMessage = call.argument("couldNotOpenWebBrowserErrorMessage")
            unexpectedErrorMessage = call.argument("unexpectedErrorMessage")
            fileDownloadErrorMessage = call.argument("fileDownloadErrorMessage")
        }

        Zowie.setStrings(zowieStrings)

        result.success()
    }

    private fun onColors(call: MethodCall, result: Result) {
        val zowieColors = ZowieColors {
            messageStatusSendingErrorColor = parseHexToColorInt(call.argument("messageStatusSendingErrorColor"))
            messageStatusDeliveredColor = parseHexToColorInt(call.argument("messageStatusDeliveredColor"))
            sentMessageBackgroundColor = parseZowieColor(call.argument("sentMessageBackgroundColor"))
            sentMessageContentsColor = parseHexToColorInt(call.argument("sentMessageContentsColor"))
            sentMessageImageUploadLoadingColor = parseHexToColorInt(call.argument("sentMessageImageUploadLoadingColor"))
            sentMessageVideoUploadLoadingColor = parseHexToColorInt(call.argument("sentMessageVideoUploadLoadingColor"))
            sentMessageImagePlaceholderLoadingColor = parseHexToColorInt(call.argument("sentMessageImagePlaceholderLoadingColor"))
            sentMessageImagePlaceholderBackgroundColor = parseHexToColorInt(call.argument("sentMessageImagePlaceholderBackgroundColor"))
            sentMessageLinksColor = parseHexToColorInt(call.argument("sentMessageLinksColor"))
            incomingMessageBackgroundColor = parseZowieColor(call.argument("incomingMessageBackgroundColor"))
            incomingMessagePrimaryTextColor = parseHexToColorInt(call.argument("incomingMessagePrimaryTextColor"))
            incomingMessageSecondaryTextColor = parseHexToColorInt(call.argument("incomingMessageSecondaryTextColor"))
            incomingMessageFileIconColor = parseZowieColor(call.argument("incomingMessageFileIconColor"))
            incomingMessageFileDownloadSuccessIconColor = parseHexToColorInt(call.argument("incomingMessageFileDownloadSuccessIconColor"))
            incomingMessageDownloadFileIconColor = parseHexToColorInt(call.argument("incomingMessageDownloadFileIconColor"))
            incomingMessageDownloadFileLoadingColor = parseHexToColorInt(call.argument("incomingMessageDownloadFileLoadingColor"))
            incomingMessageImagePlaceholderLoadingColor = parseHexToColorInt(call.argument("incomingMessageImagePlaceholderLoadingColor"))
            incomingMessageImagePlaceholderBackgroundColor = parseHexToColorInt(call.argument("incomingMessageImagePlaceholderBackgroundColor"))
            incomingMessageLinksColor = parseHexToColorInt(call.argument("incomingMessageLinksColor"))
            backgroundColor = parseHexToColorInt(call.argument("backgroundColor"))
            newMessageTextColor = parseHexToColorInt(call.argument("newMessageTextColor"))
            newMessageHintTextColor = parseHexToColorInt(call.argument("newMessageHintTextColor"))
            sendAttachmentButtonColor = parseHexToColorInt(call.argument("sendAttachmentButtonColor"))
            sendTextButtonColor = parseZowieColor(call.argument("sendTextButtonColor"))
            separatorColor = parseHexToColorInt(call.argument("separatorColor"))
            chatMessagesLoadingColor = parseHexToColorInt(call.argument("chatMessagesLoadingColor"))
            quickButtonBackgroundColor = parseHexToColorInt(call.argument("quickButtonBackgroundColor"))
            quickButtonTextColor = parseHexToColorInt(call.argument("quickButtonTextColor"))
            quickButtonPressedStrokeColor = parseHexToColorInt(call.argument("quickButtonPressedStrokeColor"))
            actionButtonBackgroundColor = parseHexToColorInt(call.argument("actionButtonBackgroundColor"))
            actionButtonBackgroundPressedColor = parseHexToColorInt(call.argument("actionButtonBackgroundPressedColor"))
            actionButtonTextColor = parseHexToColorInt(call.argument("actionButtonTextColor"))
            videoThumbnailPlaceholderColor = parseHexToColorInt(call.argument("videoThumbnailPlaceholderColor"))
            notificationErrorContentsColor = parseHexToColorInt(call.argument("notificationErrorContentsColor"))
            notificationErrorBackgroundColor = parseHexToColorInt(call.argument("notificationErrorBackgroundColor"))
            notificationSuccessContentsColor = parseHexToColorInt(call.argument("notificationSuccessContentsColor"))
            notificationSuccessBackgroundColor = parseHexToColorInt(call.argument("notificationSuccessBackgroundColor"))
            zowieLogoButtonBackgroundPressedColor = parseHexToColorInt(call.argument("zowieLogoButtonBackgroundPressedColor"))
            zowieLogoButtonPressedStrokeColor = parseHexToColorInt(call.argument("zowieLogoButtonPressedStrokeColor"))
            playVideoButtonBackgroundColor = parseHexToColorInt(call.argument("playVideoButtonBackgroundColor"))
            playVideoButtonBackgroundPressedColor = parseHexToColorInt(call.argument("playVideoButtonBackgroundPressedColor"))
            playVideoButtonPlayIconColor = parseHexToColorInt(call.argument("playVideoButtonPlayIconColor"))
        }

        Zowie.setColors(zowieColors)
        result.success()
    }

    private fun parseHexToColorInt(hex: String?): Int? = hex?.let { Color.parseColor(hex) }

    private fun parseZowieColor(jsonString: String?): ZowieColor? = jsonString?.let {
        kotlin.runCatching {
            val json = JSONObject(it)
            when (val type = json.getString("type")) {
                "solid" -> ZowieColor.Solid(parseHexToColorInt(json.getString("color"))!!)
                "gradient" -> ZowieColor.Gradient(
                        parseHexToColorInt(json.getString("startColor"))!!,
                        parseHexToColorInt(json.getString("endColor"))!!
                )
                else -> throw IllegalArgumentException("unsupported color type: $type")
            }
        }.getOrNull()
    }

    private fun getErrorListener(result: Result, code: String, message: String = "Unknown error") =
            object : ZowieOnErrorListener {
                override fun onError() = result.error(code, message)
            }

    private fun getSuccessListener(result: Result, data: Any = true) =
            object : ZowieOnSuccessListener {
                override fun onSuccess() = result.success(data)
            }

    private fun Result.success() = success(true)
    private fun Result.error(code: String, message: String) = error(code, message, "")

    companion object {
        private const val CHANNEL = "zowie_flutter_sdk"
        private const val METHOD_CONFIGURATION = "configuration"
        private const val METHOD_SHOW_CHAT = "showChat"
        private const val METHOD_METADATA = "metadata"
        private const val METHOD_ENABLE_NOTIFICATIONS = "enableNotifications"
        private const val METHOD_DISABLE_NOTIFICATIONS = "disableNotifications"
        private const val METHOD_SET_USER_STATUS = "setUserStatus"
        private const val METHOD_SET_CONTEXT = "setContext"
        private const val METHOD_LOCALIZATION = "localization"
        private const val METHOD_COLORS = "colors"
    }
}