import Flutter
import UIKit
import ZowieSDK

public class SwiftZowieFlutterSdkPlugin: NSObject, FlutterPlugin {

    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(
            name: CHANNEL,
            binaryMessenger: registrar.messenger()
        )
        
        let instance = SwiftZowieFlutterSdkPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch(call.method) {
            case SwiftZowieFlutterSdkPlugin.METHOD_CONFIGURATION:
                onConfiguration(call: call, result: result)
            case SwiftZowieFlutterSdkPlugin.METHOD_SHOW_CHAT:
                onShowChat(call: call, result: result)
            case SwiftZowieFlutterSdkPlugin.METHOD_METADATA:
                onMetadata(call: call, result: result)
            case SwiftZowieFlutterSdkPlugin.METHOD_ENABLE_NOTIFICATIONS:
                onEnableNotifications(call: call, result: result)
            case SwiftZowieFlutterSdkPlugin.METHOD_DISABLE_NOTIFICATIONS:
                onDisableNotifications(call: call, result: result)
            case SwiftZowieFlutterSdkPlugin.METHOD_SET_USER_STATUS:
                onSetUserStatus(call: call, result: result)
            case SwiftZowieFlutterSdkPlugin.METHOD_SET_CONTEXT:
                onSetContext(call: call, result: result)
            case SwiftZowieFlutterSdkPlugin.METHOD_LOCALIZATION:
                onLocalization(call: call, result: result)
            case SwiftZowieFlutterSdkPlugin.METHOD_COLORS:
                onColors(call: call, result: result)
            default:
                result(FlutterMethodNotImplemented)
        }
    }
    
    private func onConfiguration(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let arguments = call.arguments as! Dictionary<String, Any>
        let jwt = arguments["jwt"] as? String
        
        let configuration = ZowieConfiguration(
            userId: arguments["userId"] as! String,
            conversationId: arguments["conversationId"] as! String,
            instanceId: arguments["instanceId"] as! String,
            referralId: arguments["conversationInitReferral"] as? String,
            authType: (jwt?.isEmpty != false) ? .raw : .token(jwt!)
        )
        
        Zowie.shared.set(configuration: configuration)
        
        result(true)
    }

    private func onShowChat(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let rootController = UIApplication.shared.keyWindow?.rootViewController
        let navigationController = rootController?.navigationController
        let chatController = ZowieChatViewController()

        if (rootController is UINavigationController) {
            (rootController as! UINavigationController).pushViewController(chatController, animated:true)
        } else if (navigationController != nil) {
            navigationController?.pushViewController(chatController, animated: true)
        } else if (rootController != nil) {
            let navigationController:UINavigationController! = UINavigationController(rootViewController:chatController)
            rootController!.present(navigationController, animated:true, completion:nil)
        } else {
            result(false)
            return
        }

        result(true)
    }

   
    
    private func onMetadata(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let arguments = call.arguments as! Dictionary<String, Any>
        
        let metadata = ZowieMetadata(
            firstName: arguments["firstName"] as? String,
            lastName: arguments["lastName"] as? String,
            name: arguments["name"] as? String,
            locale: arguments["locale"] as? String,
            timeZone: arguments["timeZone"] as? String,
            phoneNumber: arguments["phoneNumber"] as? String,
            email: arguments["email"] as? String
        )
        
        Zowie.shared.set(metadata: metadata)
        
        result(true)
    }

    private func onEnableNotifications(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let arguments = call.arguments as! Dictionary<String, Any>
        let token = arguments["fcmToken"] as! String
        
        Zowie.shared.set(fcmToken: token) { setResult in
            switch(setResult) {
                case .success():
                    result(true)
                case .failure(_):
                    result(false)
                }
        }
    }

    private func onDisableNotifications(call: FlutterMethodCall, result: @escaping FlutterResult) {
        Zowie.shared.disableNotifications() { disableResult in
            switch(disableResult) {
                case .success():
                    result(true)
                case .failure(_):
                    result(false)
                }
        }
    }

    private func onSetUserStatus(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let arguments = call.arguments as! Dictionary<String, Any>
        let isActive = arguments["isActive"] as! Bool
        
        Zowie.shared.setStatus(isActive: isActive) { setResult in
            switch(setResult) {
                case .success():
                    result(true)
                case .failure(_):
                    result(false)
                }
        }
    }

    private func onSetContext(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let arguments = call.arguments as! Dictionary<String, Any>
        let contextId = arguments["contextId"] as! String
        
        Zowie.shared.set(contextId: contextId) { setResult in
            switch(setResult) {
                case .success():
                    result(true)
                case .failure(_):
                    result(false)
                }
        }
    }

    private func onLocalization(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let arguments = call.arguments as! Dictionary<String, Any>
        
        let strings = ZowieStrings(
            messagePlaceholder: arguments["newMessageHint"] as? String,
            sendFailureErrorMessage: arguments["messageStatusSendingErrorMessage"] as? String,
            tryAgain: arguments["messageStatusSendingErrorTryAgain"] as? String,
            delivered: arguments["messageStatusDelivered"] as? String,
            attachment: arguments["attachmentPlaceholderName"] as? String,
            disconnectMessage: arguments["chatConnectionErrorMessage"] as? String,
            reconnectMessage: arguments["chatConnectionRestoredMessage"] as? String,
            historyErrorMessage: arguments["chatHistoryDownloadErrorMessage"] as? String
        )

        Zowie.shared.set(strings: strings)
        
        result(true)
    }

    private func onColors(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let arguments = call.arguments as! Dictionary<String, Any>
        
        let colors = ZowieColors(
            background: parseHexToUIColor(hex: arguments["backgroundColor"] as? String),
            text: parseHexToUIColor(hex: arguments["newMessageTextColor"] as? String),
            textSecondary: parseHexToUIColor(hex: arguments["incomingMessageSecondaryTextColor"] as? String),
            textPlaceholder: parseHexToUIColor(hex: arguments["newMessageHintTextColor"] as? String),
            textButtonTint: parseHexToUIColor(hex: arguments["sendAttachmentButtonColor"] as? String),
            sendTextButtonTint: parseZowieColor(jsonString: arguments["sendTextButtonColor"] as? String),
            separator: parseHexToUIColor(hex: arguments["separatorColor"] as? String),
            incomingMessageBackground: parseZowieColor(jsonString: arguments["incomingMessageBackgroundColor"] as? String),
            incomingMessageTint: parseHexToUIColor(hex: arguments["incomingMessagePrimaryTextColor"] as? String),
            sentMessageBackground: parseZowieColor(jsonString: arguments["sentMessageBackgroundColor"] as? String),
            sentMessageTint: parseHexToUIColor(hex: arguments["sentMessageContentsColor"] as? String),
            incomingLinks: parseHexToUIColor(hex: arguments["incomingMessageLinksColor"] as? String),
            sentLinks: parseHexToUIColor(hex: arguments["sentMessageLinksColor"] as? String),
            deliveredLabelTint: parseHexToUIColor(hex: arguments["messageStatusDeliveredColor"] as? String),
            errorLabelTint: parseHexToUIColor(hex: arguments["messageStatusSendingErrorColor"] as? String),
            activityIndicatorTint: parseHexToUIColor(hex: arguments["chatMessagesLoadingColor"] as? String),
            sentMessageActivityIndicator: parseHexToUIColor(hex: arguments["sentMessageImagePlaceholderLoadingColor"] as? String),
            incomingMessageActivityIndicator: parseHexToUIColor(hex: arguments["incomingMessageImagePlaceholderLoadingColor"] as? String),
            videoActivityIndicator: parseHexToUIColor(hex: arguments["sentMessageVideoUploadLoadingColor"] as? String),
            quickButtonBackground: parseHexToUIColor(hex: arguments["quickButtonBackgroundColor"] as? String),
            quickButtonTint: parseHexToUIColor(hex: arguments["quickButtonTextColor"] as? String),
            quickButtonSelectedBorder: parseHexToUIColor(hex: arguments["quickButtonPressedStrokeColor"] as? String),
            actionButtonBackground: parseHexToUIColor(hex: arguments["actionButtonBackgroundColor"] as? String),
            actionButtonSelectedBackground: parseHexToUIColor(hex: arguments["actionButtonBackgroundPressedColor"] as? String),
            actionButtonTint: parseHexToUIColor(hex: arguments["actionButtonTextColor"] as? String),
            actionButtonShadow: parseHexToUIColor(hex: arguments["actionButtonShadow"] as? String),
            fileIconTint: parseZowieColor(jsonString: arguments["incomingMessageFileIconColor"] as? String),
            downloadFileIcon: parseHexToUIColor(hex: arguments["incomingMessageDownloadFileIconColor"] as? String),
            fileSuccessIconTint: parseHexToUIColor(hex: arguments["incomingMessageFileDownloadSuccessIconColor"] as? String),
            videoPlaceholder: parseHexToUIColor(hex: arguments["videoThumbnailPlaceholderColor"] as? String),
            videoButtonBackground: parseHexToUIColor (hex: arguments["playVideoButtonBackgroundColor"] as? String),
            videoButtonIconTint: parseHexToUIColor(hex: arguments["playVideoButtonPlayIconColor"] as? String),
            logoSelectedBackground: parseHexToUIColor(hex: arguments["zowieLogoButtonBackgroundPressedColor"] as? String),
            logoSelectedBorder: parseHexToUIColor(hex: arguments["zowieLogoButtonPressedStrokeColor"] as? String),
            notificationFailureTint: parseHexToUIColor(hex: arguments["notificationErrorContentsColor"] as? String),
            notificationSuccessTint: parseHexToUIColor(hex: arguments["notificationSuccessContentsColor"] as? String),
            notificationFailureBackground: parseHexToUIColor(hex: arguments["notificationErrorBackgroundColor"] as? String),
            notificationSuccessBackground: parseHexToUIColor(hex: arguments["notificationSuccessBackgroundColor"] as? String)
        )
        
        Zowie.shared.set(colors: colors)
        
        result(true)
    }
    
    private func parseHexToUIColor(hex: String?) -> UIColor? {
        guard let hex = hex else { return nil }
        return UIColor(hexString: hex)
    }
    
    private func parseZowieColor(jsonString: String?) -> ZowieColor? {
        guard let jsonString = jsonString else { return nil }
        
        do {
            let json = try JSONSerialization.jsonObject(with: Data(jsonString.utf8), options: []) as! [String: Any]
            let type = json["type"] as! String
            switch(type) {
                case "solid":
                    return ZowieColor.solid(parseHexToUIColor(hex: json["color"] as? String)!)
                case "gradient":
                    return ZowieColor.gradient(
                        start: parseHexToUIColor(hex: json["startColor"] as? String)!,
                        end: parseHexToUIColor(hex: json["endColor"] as? String)!
                    )
                default:
                    print("Error, parseZowieColor, unsupported color type \(type)")
            }
        } catch let error as NSError {
            print("Error, parseZowieColor, \(error.localizedDescription)")
        }

        return nil
    }
    
    private static let CHANNEL = "zowie_flutter_sdk"
    private static let METHOD_CONFIGURATION = "configuration"
    private static let METHOD_SHOW_CHAT = "showChat"
    private static let METHOD_METADATA = "metadata"
    private static let METHOD_ENABLE_NOTIFICATIONS = "enableNotifications"
    private static let METHOD_DISABLE_NOTIFICATIONS = "disableNotifications"
    private static let METHOD_SET_USER_STATUS = "setUserStatus"
    private static let METHOD_SET_CONTEXT = "setContext"
    private static let METHOD_LOCALIZATION = "localization"
    private static let METHOD_COLORS = "colors"
    
  }

extension UIColor {
    convenience init(hexString: String) {
        let hex = hexString.trimmingCharacters(in: CharacterSet.alphanumerics.inverted)
        var int = UInt64()
        Scanner(string: hex).scanHexInt64(&int)
        let a, r, g, b: UInt64
        switch hex.count {
        case 3:
            (a, r, g, b) = (255, (int >> 8) * 17, (int >> 4 & 0xF) * 17, (int & 0xF) * 17)
        case 6:
            (a, r, g, b) = (255, int >> 16, int >> 8 & 0xFF, int & 0xFF)
        case 8:
            (a, r, g, b) = (int >> 24, int >> 16 & 0xFF, int >> 8 & 0xFF, int & 0xFF)
        default:
            (a, r, g, b) = (255, 0, 0, 0)
        }
        self.init(red: CGFloat(r) / 255, green: CGFloat(g) / 255, blue: CGFloat(b) / 255, alpha: CGFloat(a) / 255)
    }
}
